// Включаем библиотеку для работы с портами ввода-вывода микроконтроллера AVR.
#include <avr/io.h>
// Определяем частоту процессора микроконтроллера в 1 МГц.
#define F_CPU 1000000UL
// Определяем бит в регистре PIND для сигнала с клока энкодера.
#define PIND_CLK 0b00010000
// Определяем бит в регистре PIND для сигнала с DT энкодера.
#define PIND_DT 0b01000000

volatile uint8_t encoder_state, prev_encoder_state, state;
int sw34, pin_clk_2, pin_dt_2;


void read_encoder_state()
{
    // Устанавливаем начальные значения переменных равное 0
    pin_clk_2 = 0;
    pin_dt_2 = 0;

    // Устанавливаем значение PIN_CLK_2 в зависимости от установки бита PIND_CLK в PIND
    pin_clk_2 = (PIND & PIND_CLK) ? 1 : 0;
    // Устанавливаем значение PIN_DT_2 в зависимости от установки бита PIND_DT в PIND
    pin_dt_2 = (PIND & PIND_DT) ? 1 : 0;
    // Получаем состояние энкодера
    encoder_state = (pin_clk_2 << 1) | pin_dt_2;

    // Проверяем, изменилось ли состояние энкодера
    if (encoder_state != prev_encoder_state) 
    {
        switch ((prev_encoder_state << 2) | encoder_state) 
        {
            case 1: // поворот по часовой стрелке
                state++;
                break;
            case 2: // поворот против часовой стрелке
                state--;
                break;
        }

        // Устанавливаем значение prev_encoder_state равным текущему состоянию энкодера
        prev_encoder_state = encoder_state;
    }

}

void encoder_setup()
{
    // Инициализация последовательного порта
    Serial.begin(9600);

    // Настройка портов ввода-вывода
    DDRD = 0x00;
    PORTD = (PIND_CLK + PIND_DT);
}

uint8_t encoder_value()
{
    // Читаем состояние энкодера
    read_encoder_state();

    // Если состояние энкодера изменилось, выводим его в последовательный порт
    if (state != sw34)
    {
        //Serial.println(state);
        sw34 = state;
        return state;
    }
}